﻿:namespace newcmd
⍝ This class will help create a new Spice/user command V1.02

⍝ It will first create a form to gather basic info about the Spice commands
⍝ to create then create the frame of the namespace which can be edited before saving it
⍝ as a Spice command.
    ⎕IO←1 ⋄ ⎕ML←1

    ∇ r←List
      r←⎕NS¨1⍴⊂⍬
    ⍝ Name, group, short description and parsing rules
      r.Name←⊂'UNew'
      r.Group←⊂'Spice'
      r[1].Desc←'Create a new User Command interactively'
      r[1].Parse←''
    ∇

    ∇ r←Run(Cmd Args)
      NewCmd Args
      r←0 0⍴''
    ∇

    ∇ r←Help Cmd
      r←⊂'This command will ask basic details (name, description) about a new'
      r,←⊂'User command and create a new class skeleton that can be further edited'
      r,←⊂'and filed to be tested. Use F1 for details when running it.'
      r←↑r
    ∇

    ∇ NewCmd name;Cmds;Form1
    ⍝ Enter command details thru a form them make it real
      FORM←MAKE_Form
      Cmds←⍬
      FORM FILL_Form name'MyCmds'
      ⎕DQ FORM      ⍝ several commands can be defined here
      MAKE_Class Cmds
      ⎕EX FORM
    ∇

    ∇ tn←MAKE_Switches mat;st
    ⍝ Make switches descriptions
      tn←'-',¨mat[;1]~¨' '                    ⍝ name
      st←'=∊'[1+1∊¨mat[;4]]                   ⍝ parameter style
      st←st{(⍵/'['),⍺,⍵/']'}¨'M'∊¨mat[;2]     ⍝ add [] if 'maybe'
      tn←⍕tn,¨(~'N'∊¨mat[;2])/¨st,¨mat[;3]    ⍝ add validation parameters
    ∇

    ∇ MAKE_Class cmdlist;src;qu;in;t;i;name;fname;F;cmds;newname;prev;edr;same;now
     ⍝ Create a namespace template from a list of commands descriptions
      →(⍴cmdlist)↓0
      qu←{Q,⍨Q,(1+⍵=Q←'''')/⍵}
     ⍝ Find a name for the namespace in the ws
      src←⍴i←0
      :While 0≠#.⎕NC name←'myCmdGrp',(0≠i)/⍕i ⋄ i+←1 ⋄ :EndWhile
     ⍝ Create the namespace source
      src,←⊂':Namespace ',name
      src,←'⍝ Private User Command' '' '⎕IO←1 ⋄ ⎕ML←1' ''
      src,←⊂'∇ r←List'
      src,←⊂'r←⎕NS¨',(⍕⍴cmdlist),'⍴⊂⍬'
      src,←⊂'   ⍝ Name, group, short description and parsing rules'
      src,←⊂'r.Name←',(2×1∧.<∊⍴¨t)↓',¨',{1∊⍴c←∪⍵:'⊂',⍕c ⋄ ⍕⍵}qu¨cmds←' '~⍨¨t←1⊃¨cmdlist
      src,←⊂'r.Group←',{1∊⍴c←∪⍵:'⊂',⍕c ⋄ ⍕⍵}qu¨2⊃¨cmdlist
    ⍝ Put in short descriptions
      t←↓'<r[>,I3,<].Desc←>'⎕FMT in←⍳⍴cmdlist
      src,←t,¨qu¨5⊃¨cmdlist
    ⍝ Parsing rules
    ⍝ The rules are made up of the ## of args followed by the switches description
      t←MAKE_Switches¨4⊃¨cmdlist ⍝ a table of switch name, style, params & param type
      t,⍨¨←3⊃¨cmdlist
      src,←⊂'r.Parse←',{1∊⍴c←∪⍵:'⊂',⍕c ⋄ ⍕⍵}qu¨t
      src,←'∇ ' ''
     
      src,←'∇ r←Run(Cmd Input)' ':Select Cmd'
      :For i :In in
          src,←⊂':Case ',(1<⍴t)↓',',qu t←i⊃cmds
          src,←⊂'r←',(qu'command ',t,' in construction'),' ⍝ ENTER COMMAND CODE HERE'
      :EndFor
      src,←':EndSelect' '∇ ' ''
     
      src,←'∇ r←level Help Cmd;⎕ml' '⎕ml←1' ':Select Cmd'
      :For i :In in
          src,←⊂':Case ',qu i⊃cmds
          :If 1∊⍴t←qu¨i 6⊃cmdlist
              src,←⊂'r←',⍕t
          :Else
              src,←⊂'r←⍬'
              src,←'r,←⊂'∘,¨t
              src,←⊂'r←↑r'
          :EndIf
      :EndFor
      src,←':EndSelect' '∇ ' '' ':EndNamespace'
     
      newname←name←⍕#.⎕FIX src ⋄ prev←#.⎕NL-9 ⋄ edr←⎕ED name ⋄ same←prev≡now←#.⎕NL-9
      :If same∧0∊⍴edr ⍝ we already have the answer
      :ElseIf ~same
          newname←'#.',⊃now~prev
      :Else
          newname←'#.',edr
      :EndIf
     
      :If 1∊'Query'Msg('Namespace ',newname,' ready')'Save it?'
          t←''⎕SE.SALTUtils.ClassFolder⍨(∧\';'≠t)/t←⎕SE.SALT.Settings'cmddir'
          'F'⎕WC'FileBox' 'Save new command script't'*.dyalog'('Event' 71 1)
      :AndIf 71∊2⊃2↑t←⎕DQ'F'
          ⎕SE.SALT.Save newname,' ',3⊃t
      :Else
          (qu newname),' NOT saved.'
      :EndIf
    ∇

    ∇ fname FILL_Form namgrp;f1
    ⍝ Fill the form with default values
      :With fname
          sdt.Text←sdt.Tip←'Text to appear on ]?cmd here'
          ldt.Text←ldt.Tip←'Enter text to appear on ]??cmd in this box'
          namet.Text grpt.Text←namgrp
          Group0.nat.Text←''
          Group0.nat.Value←⍬ ⍝ because it is not reset when Text is assigned
          Group0.(max long).State←0
          Group1.G.Values←4 4⍴⊂''
      :EndWith
      f1←fname,'.',(1+' '∧.=1⊃namgrp)⊃'grpt' 'namet'
      ⎕NQ f1'GotFocus'
    ∇

    ∇ {fname}←MAKE_Form;h;lm;l1;fh;fw;fhw;mw;bsz;∆;ns;⎕ML;CR;Here;cbf;font;evt;t;l2;l3;l4;l5;color
    ⍝ Create the form to enter new Spice command info
      ⎕ML←1 ⋄ CR←⎕AV[4] ⋄ Here←⍕⎕THIS
    ⍝ Basic form globals
      h←24 ⋄ lm←16 ⋄ l5←70+l4←150+l3←50+l2←30+l1←20
      fh fw←fhw←530 450 ⋄ color←255 150 150 ⋄ mw←fw-2×lm
      :With fname←'Form1'⎕WC'Form' 'Define New User Command'(142 220)fhw('Coord' 'Pixel')('EdgeStyle' 'Dialog')('Border' 2)
          TipObj←'tf'⎕WC'TipField'
          font←(1+80=⎕DR'')⊃'Dyalog Alt' 'APL385 Unicode'
          font←'Font'font 16 1 0 0 400 0  ⍝ Dyalog 16 normal
          ⎕WS evt←'event' 22(Here,'.ncHelp')
          'name'⎕WC'Label' 'Name:'(l1 lm)
          'namet'⎕WC'Edit' ''((l1+1)60)(h 120)evt font
          namet.Tip←'Enter the name of the UCMD here'
          'grp'⎕WC'Label' 'Group:'(l1 190)
          'grpt'⎕WC'Edit' ''((l1+1)240)(h 80)evt font
          grpt.Tip←'Enter the name of the group to which the UCMD belongs'
          'sd'⎕WC'Label' 'Short description:'((l2+2)lm)
          'sdt'⎕WC'Edit' ''((l2+25)lm)(h mw)('HScroll' 0)evt
          'ld'⎕WC'Label' 'Long description:'((l3+2)lm)
          'ldt'⎕WC'Edit' ''((l3+25)lm)(110 mw)'multi'evt
     
          :With 'Group0'⎕WC'Group' 'Arguments'(l4 lm)(60 mw)evt
              gl2←7+gl1←15 ⋄ gp2←65+gp1←160
              'na'⎕WC'Label' '# of args:'(gl2 20)
              'nat'⎕WC'Edit' ''((gl2+1)105)(h 32)('FieldType' 'Numeric')evt('ValidIfEmpty' 1)
              'max'⎕WC'Button' 'Number is upper limit:'(gl1 gp1)(h gp2)('Align' 'Left')('Style' 'Check')
              'long'⎕WC'Button' 'Merge trailing arguments:'((gl1+20)gp1)(h gp2)('Align' 'Left')('Style' 'Check')
              nat.Tip←'Enter a value here if you wish to limit the number of arguments that the UCMD should accept'
              max.Tip←'Check this box if the number of arguments specified is the maximum number of arguments allowed'
              long.Tip←'Check this box to merge values provided for arguments beyond the specified number into a single value'
          :EndWith
     
          :With 'Group1'⎕WC'Group' 'Switches'(l5 lm)(163 mw)evt
              'G'⎕WC'Grid' ⋄ ns←4 ⋄ t←'event' 22(Here,'.ncHelp')
              'G.e'⎕WC'Edit'('EdgeStyle' 'Recess')t
              'G.c'⎕WC'Combo'('No' 'Yes' 'Maybe')t
     
              'G.b'⎕WC'button' ''('Style' 'Check')t
              'G'⎕WS'CellHeights'h
              'G'⎕WS'CellSelect' 'Whole'
              'G'⎕WS'CellTypes'(ns 4⍴1 2 1 3)
              'G'⎕WS'CellWidths'(⌊75 67 130 33×1.2)
              'G'⎕WS'TitleHeight'(2×h)
              'G'⎕WS'ColTitles'('Name'('Accepts',CR,'values')('Acceptable',CR,'values')('As a',CR,'set'))
              'G'⎕WS'AutoExpand' 1 0
              'G'⎕WS'Input'('G.'∘,¨'ecb')
              'G'⎕WS'Posn'(20 20)
              'G'⎕WS'ResizeCols'(1 0 1 0)
              'G'⎕WS'Size'(133,+/21,G.CellWidths)
              'G'⎕WS'TitleWidth' 0
              'G'⎕WS'Values'(ns 4⍴⊂'')
          :EndWith
          cbf←Here,'.NextCmd'
          'next'⎕WC'button' 'Next Command'(490 lm)('event' 'select'cbf 1)
          'done'⎕WC'button' 'Done'(490,5+2⊃next.(Size+Posn))('event' 'select'cbf 0)
          'cancel'⎕WC'button' 'Cancel'(490,5+2⊃done.(Size+Posn))('cancel' 1)('event' 'select' 1)
      :EndWith
    ∇

    ∇ ncHelp msg;f;sz;txt;evt
      :If 'F1'≡3⊃msg
          evt←'event' 22 1 ⍝ terminate on ANY char
          'f'⎕WC'form' 'Help for newcmd'(100 500)(sz←730 630)('coord' 'pixel')evt
          'f.t'⎕WC'edit'HELP(0 0)(sz-1)'multi'evt
          ⎕DQ'f'
      :EndIf
    ∇
    
    CurrentCmds←{6::0 ⋄ ⎕se.SALTUtils.lCase¨⎕SE.cbbot.bandsalt.Spice.List[;1]}0

    ∇ more NextCmd arg;ng;gdata;any;t;anytxt;Msg;val
    ⍝ Callback fn for the 'Next' button on Form1
    ⍝ Verify entered data. 'more' means 'keep entering data if checks succeed'
      :With FORM
          Msg←##.Msg
          any←∨/''∘≢¨gdata←Group1.G.Values ⋄ ng←namet.Text grpt.Text
          {}÷'stop'≢namet.Text
          :If more∨(1∊any)∨∨/anytxt←' '∨.≠¨namet.Text Group0.nat.Text sdt.Text,⊂∊ldt.Text   ⍝ any new fields?
              :If ¯1∊∊⎕NC¨ng
                  Msg'Invalid name or group' 'Enter a valid APL name'
                  ⎕NQ'namet' 'GotFocus'
                  →0
              :ElseIf (⎕SE.SALTUtils.lCase¨ng[1]~¨' ')∊##.CurrentCmds
                  Msg'This command already exists' 'Enter another name'
                  ⎕NQ'namet' 'GotFocus'
                  →0
              :ElseIf 1↑0≥Group0.nat.Value
                  Msg'Invalid number of arguments' 'Enter a number >0'
                  ⎕NQ'Group0.nat' 'GotFocus'
                  →0
              :ElseIf ∨/t←¯1=any/∊⎕NC¨0 ¯3↓gdata
                  Msg'Invalid switch name' 'Enter a valid APL name'
                  ⎕NQ'Group1.G' 'GotFocus' ⍝ which t?
                  →0
              :ElseIf t≢∪t←any/gdata[;1]
                  Msg'Switches names not unique' 'Rename same name switches'
                  →0
              :ElseIf ∨/any/(gdata[;4]≡¨1)∧0∊¨⍴¨gdata[;3]
                  Msg'Invalid SET values' 'Enter at least ONE character of the set to use'
                  →0
              :ElseIf ~anytxt[3]
                  Msg'Short description mandatory' 'Enter a short description for this command'
                  ⎕NQ'sdt' 'GotFocus'
                  →0
              :ElseIf ~anytxt[4]
                  Msg'Long description mandatory' 'Enter a proper description for this command'
                  ⎕NQ'ldt' 'GotFocus'
                  →0
              :Else ⍝ looks fine, gather cmd data
                  t←Group0.(max long).State ⋄ val←t{0∊⍴⍵:⍕1∩⍺ ⋄ ⍵}⍕Group0.nat.Value
                  ##.Cmds,←⊂ng,(⊂val,t/'SL'),(any⌿gdata)sdt.Text ldt.Text
              :EndIf
          :EndIf
      :EndWith
      :If more
          FORM FILL_Form' ',2⌷ng
      :Else
          ⎕EX FORM ⍝ Erase Form1 here to end this jig
      :EndIf
    ∇

    ∇ {r}←{style}Msg(cap txt);mb;evt
      txt←⍕(⌈/18 0+⍴¨cap txt)↑¨⊂txt
      :If 0=⎕NC'style' ⋄ style←'Msg' ⋄ :EndIf
      'mb'⎕WC'msgbox'cap txt style('Event'(evt←61 62 63)1)
      r←4|evt⍳(⎕DQ'mb')[2]
    ∇

    _←⍬
    _,←'NAME: Enter the name of a new command.' 'This field is compulsory.'
    _,←'It must follow the rules of a valid APL name.'  ''

    _,←⊂'GROUP: Enter the name of a group this command will belong to.'
    _,←⊂'For example, if this command relates to MATH then enter'
    _,←⊂'MATH or something to remind you of what the command relates to.'
    _,←'The command does NOT have to belong to a group but it is' 'a good idea to make it so.' ''

    _,←,⊂'SHORT DESCRIPTION: Spice needs a short description to be able to display '
    _,←⊂'briefly what the command does when you type ]?'
    _,←'This is a one-liner you need to put here.' ''

    _,←,⊂'LONG DESCRIPTION: Spice needs a fuller description of what the command does'
    _,←⊂'to be able to display it when you type    ]? mycmd'
    _,←'This is free format text you need to put here.' ''

    _,←⊂'# ARGS: Enter the number of arguments this command accepts.'
    _,←⊂'Arguments are space delimited words or numbers'
    _,←⊂'(if a space is part of the argument put quotes around it).'
    _,←'If it is unknown you can leave it blank.' 'If there is a maximum then enter it and'
    _,←,'tick the ''Max'' box.' ''

    _,←'NUMBER is upper limit: When this box is ticked the command will' 'accept fewer arguments than the number specified.'
    _,←,⊂'For example if the number of arguments is 3 and this box'
    _,←'is ticked then the user will be able to enters' 'less than 3, even none.' ''

    _,←'MERGE trailing args: When this box is ticked the' 'last argument will contain whatever remains.'
    _,←,⊂'For example if the number of arguments is 3 and this box'
    _,←'is ticked and the user enters:' 'ONE 2 3 Four'
    _,←,⊂'The first argument will be ''ONE'', the second will be ''2'''
    _,←,'and the third will be ''3 Four''' ''

    _,←'SWITCHES: The switches are made up of up to 4 elements:' '- the switch name, a valid APL id'
    _,←'- whether the switch takes a value or not' '- the possible values the switch accepts'
    _,←'- or the set of possible characters it accepts' ''
    _,←'For example if switch ABC accepts values ''123'' and ''xyz''' 'you would enter'
    _,←'ABC | Yes | 123 xyz ' 'if it accepts only vowels you would enter'
    _,←,'ABC | Yes | aeiou | Yes' ''

    HELP←↑_ 
    ⎕ex'_'

:Endnamespace